"""
Measurement channels module for Zigbee Home Automation.

For more details about this component, please refer to the documentation at
https://home-assistant.io/components/zha/
"""
import logging

_LOGGER = logging.getLogger(__name__)
